#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re

def getAvailableShips(s):
	"""Function that returns the total number of free (available) ships
	Parameters
	----------
	s : Session
		Session object
	
	Returns
	-------
	ships : int
		number of currently available ships
	"""
	html = s.get()
	return int(re.search(r'GlobalMenu_freeTransporters">(\d+)<', html).group(1))

def getTotalShips(s):
	"""Function that returns the total number of ships, regardless of if they're available or not
	Parameters
	----------
	s : Session
		Session object
	
	Returns
	-------
	ships : int
		total number of ships the player has
	"""
	html = s.get()
	return int(re.search(r'maxTransporters">(\d+)<', html).group(1))



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import json
import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.signals import setInfoSignal

t = gettext.translation('activateMiracle',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def obtainMiraclesAvailable(s):
	idsIslands = getIdsOfIslands(s)
	islands = []
	for idIsland in idsIslands:
		html = s.get(urlIsla + idIsland)
		isla = getIsla(html)
		isla['activable'] = False
		islands.append(isla)

	ids, citys = getIdsOfCities(s)
	for cityId in citys:
		city = citys[cityId]
		# get the wonder for this city
		wonder = [ island['wonder'] for island in islands if city['coords'] == '[{}:{}] '.format(island['x'], island['y']) ][0]
		# if the wonder is not new, continue
		if wonder in [ island['wonder'] for island in islands if island['activable'] ]:
			continue

		html = s.get(urlCiudad + str(city['id']))
		city = getCiudad(html)

		# make sure that the city has a temple
		for i in range( len( city['position'] ) ):
			if city['position'][i]['building'] == 'temple':
				city['pos'] = str(i)
				break
		else:
			continue

		# get wonder information
		params = {"view": "temple", "cityId": city['id'], "position": city['pos'], "backgroundView": "city", "currentCityId": city['id'], "actionRequest": s.token(), "ajax": "1"}
		data = s.post(params=params)
		data = json.loads(data, strict=False)
		data = data[2][1]
		available =  data['js_WonderViewButton']['buttonState'] == 'enabled'
		if available is False:
			for elem in data:
				if 'countdown' in data[elem]:
					enddate     = data[elem]['countdown']['enddate']
					currentdate = data[elem]['countdown']['currentdate']
					break

		# set the information on the island which wonder we can activate
		for isla in islands:
			if isla['id'] == city['islandId']:
				isla['activable'] = True
				isla['ciudad'] = city
				isla['available'] = available
				if available is False:
					isla['available_in'] = enddate - currentdate
				break

	# only return island which wonder we can activate
	return [ island for island in islands if island['activable'] ]

def activateMiracleImpl(s, isla):
	params = {'action': 'CityScreen', 'cityId': isla['ciudad']['id'], 'function': 'activateWonder', 'position': isla['ciudad']['pos'], 'backgroundView': 'city', 'currentCityId': isla['ciudad']['id'], 'templateView': 'temple', 'actionRequest': s.token(), 'ajax': '1'}
	rta = s.post(params=params)
	return json.loads(rta, strict=False)

def chooseIsland(islands):
	print(_('Which miracle do you want to activate?'))
	i = 0
	print(_('(0) Exit'))
	for island in islands:
		i += 1
		if island['available']:
			print('({:d}) {}'.format(i, island['wonderName']))
		else:
			print(_('({:d}) {} (available in: {})').format(i, island['wonderName'], daysHoursMinutes(island['available_in'])))

	index = read(min=0, max=i)
	if index == 0:
		return None
	island = islands[index - 1]
	return island

def activateMiracle(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		banner()

		islands = obtainMiraclesAvailable(s)
		if islands == []:
			print(_('There are no miracles available.'))
			enter()
			e.set()
			return

		island = chooseIsland(islands)
		if island is None:
			e.set()
			return

		if island['available']:
			print(_('\nThe miracle {} will be activated').format(island['wonderName']))
			print(_('Proceed? [Y/n]'))
			r = read(values=['y', 'Y', 'n', 'N', ''])
			if r.lower() == 'n':
				e.set()
				return

			rta = activateMiracleImpl(s, island)

			if rta[1][1][0] == 'error':
				print(_('The miracle {} could not be activated.').format(island['wonderName']))
				enter()
				e.set()
				return

			data = rta[2][1]
			for elem in data:
				if 'countdown' in data[elem]:
					enddate     = data[elem]['countdown']['enddate']
					currentdate = data[elem]['countdown']['currentdate']
					break
			wait_time = enddate - currentdate

			print(_('The miracle {} was activated.').format(island['wonderName']))
			enter()
			banner()

			while True:
				print(_('Do you wish to activate it again when it is finished? [y/N]'))

				r = read(values=['y', 'Y', 'n', 'N', ''])
				if r.lower() != 'y':
					e.set()
					return

				iterations = read(msg=_('How many times?: '), digit=True, min=0)

				if iterations == 0:
					e.set()
					return

				duration = wait_time * iterations

				print(_('It will finish in:{}').format(daysHoursMinutes(duration)))

				print(_('Proceed? [Y/n]'))
				r = read(values=['y', 'Y', 'n', 'N', ''])
				if r.lower() == 'n':
					banner()
					continue
				break
		else:
			print(_('\nThe miracle {} will be activated in {}').format(island['wonderName'], daysHoursMinutes(island['available_in'])))
			print(_('Proceed? [Y/n]'))
			rta = read(values=['y', 'Y', 'n', 'N', ''])
			if rta.lower() == 'n':
				e.set()
				return
			wait_time = island['available_in']
			iterations = 1

			print(_('\nThe mirable will be activated.'))
			enter()
			banner()

			while True:
				print(_('Do you wish to activate it again when it is finished? [y/N]'))

				r = read(values=['y', 'Y', 'n', 'N', ''])
				again = r.lower() == 'y'
				if again is True:
					try:
						iterations = read(msg=_('How many times?: '), digit=True, min=0)
					except KeyboardInterrupt:
						iterations = 1
						break

					if iterations == 0:
						iterations = 1
						break

					iterations += 1
					duration = wait_time * iterations
					print(_('It is not possible to calculate the time of finalization. (at least: {})').format(daysHoursMinutes(duration)))
					print(_('Proceed? [Y/n]'))

					try:
						r = read(values=['y', 'Y', 'n', 'N', ''])
					except KeyboardInterrupt:
						iterations = 1
						break

					if r.lower() == 'n':
						iterations = 1
						banner()
						continue
				break
	except KeyboardInterrupt:
		e.set()
		return

	set_child_mode(s)
	e.set()

	info = _('\nI activate the miracle {} {:d} times\n').format(island['wonderName'], iterations)
	setInfoSignal(s, info)
	try:
		do_it(s, island, iterations)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(s, msg)
	finally:
		s.logout()

def wait_for_miracle(s, isla):
	while True:
		params = {"view": "temple", "cityId": isla['ciudad']['id'], "position": isla['ciudad']['pos'], "backgroundView": "city", "currentCityId": isla['ciudad']['id'], "actionRequest": s.token(), "ajax": "1"}
		data = s.post(params=params)
		data = json.loads(data, strict=False)
		data = data[2][1]

		for elem in data:
			if 'countdown' in data[elem]:
				enddate     = data[elem]['countdown']['enddate']
				currentdate = data[elem]['countdown']['currentdate']
				wait_time = enddate - currentdate
				break
		else:
			available = data['js_WonderViewButton']['buttonState'] == 'enabled'
			if available:
				return
			else:
				wait_time = 60

		msg = _('I wait {:d} seconds to activate the miracle {}').format(wait_time, isla['wonderName'])
		sendToBotDebug(s, msg, debugON_activateMiracle)
		wait(wait_time + 5)

def do_it(s, isla, iterations):

	for i in range(iterations):

		wait_for_miracle(s, isla)

		rta = activateMiracleImpl(s, isla)

		if rta[1][1][0] == 'error':
			msg = _('The miracle {} could not be activated.').format(isla['wonderName'])
			sendToBot(s, msg)
			return

		msg = _('Miracle {} successfully activated').format(isla['wonderName'])
		sendToBotDebug(s, msg, debugON_activateMiracle)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
from ikabot.config import *
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.pedirInfo import getIdsOfCities

def getCommertialCities(s):
	ids = getIdsOfCities(s)[0]
	ciudades_comerciales = []
	for idCiudad in ids:
		html = s.get(urlCiudad + idCiudad)
		ciudad = getCiudad(html)
		for pos, edificio in enumerate(ciudad['position']):
			if edificio['building'] == 'branchOffice':
				ciudad['pos'] = pos
				html = getStoreHtml(s, ciudad)
				rangos = re.findall(r'<option.*?>(\d+)</option>', html)
				ciudad['rango'] = int(rangos[-1])
				ciudades_comerciales.append(ciudad)
				break
	return ciudades_comerciales

def getStoreHtml(s, ciudad):
	url = 'view=branchOffice&cityId={}&position={:d}&currentCityId={}&backgroundView=city&actionRequest={}&ajax=1'.format(ciudad['id'], ciudad['pos'], ciudad['id'], s.token())
	data = s.post(url)
	json_data = json.loads(data, strict=False)
	return json_data[1][1][1]

def getstorageCapacityDeVenta(html):
	match = re.search(r'var\s*storageCapacity\s*=\s*(\d+);', html)
	if match:
		return int(match.group(1))
	else:
		return 0

def vendiendo(html):
	mad, vin, mar, cri, azu = re.findall(r'<input type="text" class="textfield"\s*size="\d+"\s*name=".*?"\s*id=".*?"\s*value="(\d+)"', html)
	return [int(mad), int(vin), int(mar), int(cri), int(azu)]



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import os
import sys
import json
import time
import random
import getpass
import gettext
import requests
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.gui import banner
from ikabot.helpers.aesCipher import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.getJson import getCiudad

t = gettext.translation('sesion', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext


class Sesion:
	def __init__(self):
		self.padre = True
		self.logged = False
		self.__login()

	def __genRand(self):
		return hex(random.randint(0, 65535))[2:]

	def __genCookie(self):
		return self.__genRand() + self.__genRand() + hex(int(round(time.time() * 1000)))[2:] + self.__genRand() + self.__genRand()

	def __fp_eval_id(self):
		return self.__genRand() + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + '-' + self.__genRand() + self.__genRand() + self.__genRand()

	def __logout(self, html):
		if html is not None:
			idCiudad = getCiudad(html)['id']
			token = re.search(r'actionRequest"?:\s*"(.*?)"', html).group(1)
			urlLogout = 'action=logoutAvatar&function=logout&sideBarExt=undefined&startPageShown=1&detectedDevice=1&cityId={0}&backgroundView=city&currentCityId={0}&actionRequest={1}'.format(idCiudad, token)
			self.s.get(self.urlBase + urlLogout)

	def __isInVacation(self, html):
		return 'nologin_umod' in html

	def __isExpired(self, html):
		return 'index.php?logout' in html

	def __updateCookieFile(self, primero=False, nuevo=False, salida=False):
		msg = _('Updating the cookie file:\n')
		if primero:
			msg += _('First')
		elif nuevo:
			msg += _('New')
		else:
			msg += _('Out')
		#sendToBotDebug(self, msg, debugON_session)

		sessionData = self.getSessionData()

		if primero is True:
			cookie_dict = dict(self.s.cookies.items())
			sessionData['cookies'] = cookie_dict
			sessionData['num_sesiones'] = 1

		elif nuevo is True:
			try:
				sessionData['num_sesiones'] += 1
			except KeyError:
				sessionData['num_sesiones'] = 1

		elif salida is True:
			try:
				if sessionData['num_sesiones'] == 1:
					html = self.s.get(self.urlBase).text
					if self.__isExpired(html) is False:
						self.__logout(html)
			except KeyError:
				return
			sessionData['num_sesiones'] -= 1

		self.setSessionData(sessionData)

	def __getCookie(self, sessionData=None):
		if sessionData is None:
			sessionData = self.getSessionData()
		try:
			assert sessionData['num_sesiones'] > 0
			cookie_dict = sessionData['cookies']
			self.s = requests.Session()
			self.s.proxies = proxyDict
			self.s.headers.clear()
			self.s.headers.update(self.headers)
			requests.cookies.cookiejar_from_dict(cookie_dict, cookiejar=self.s.cookies, overwrite=True)
			self.__updateCookieFile(nuevo=True)
		except (KeyError, AssertionError):
			msg = _('Updating cookie using cookie file')
			sendToBotDebug(self, msg, debugON_session)
			self.__login(3)

	def __login(self, retries=0):
		if not self.logged:
			banner()

			self.mail = read(msg=_('Mail:'))
			self.password = getpass.getpass(_('Password:'))

			banner()

		self.s = requests.Session()
		self.s.proxies = proxyDict

		# get gameEnvironmentId and platformGameId
		self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://lobby.ikariam.gameforge.com/config/configuration.js')

		js = r.text
		gameEnvironmentId = re.search(r'"gameEnvironmentId":"(.*?)"', js)
		if gameEnvironmentId is None:
			exit('gameEnvironmentId not found')
		gameEnvironmentId = gameEnvironmentId.group(1)
		platformGameId = re.search(r'"platformGameId":"(.*?)"', js)
		if platformGameId is None:
			exit('platformGameId not found')
		platformGameId = platformGameId.group(1)

		# get __cfduid cookie
		self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://gameforge.com/js/connect.js')
		html = r.text
		captcha = re.search(r'Attention Required', html)
		if captcha is not None:
			exit('Captcha error!')

		# update __cfduid cookie
		self.headers = {'Host': 'gameforge.com', 'User-Agent': user_agent, 'Accept': '*/*', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://gameforge.com/config')

		__fp_eval_id_1 = self.__fp_eval_id()
		__fp_eval_id_2 = self.__fp_eval_id()

		# get pc_idt cookie
		self.headers = {'Host': 'pixelzirkus.gameforge.com', 'User-Agent': user_agent, 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Content-Type': 'application/x-www-form-urlencoded', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Upgrade-Insecure-Requests': '1'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		data = {'product': 'ikariam', 'server_id': '1', 'language': 'en', 'location': 'VISIT', 'replacement_kid': '', 'fp_eval_id': __fp_eval_id_1, 'page': 'https%3A%2F%2Flobby.ikariam.gameforge.com%2F', 'referrer': '', 'fingerprint': '2175408712', 'fp_exec_time': '1.00'}
		r = self.s.post('https://pixelzirkus.gameforge.com/do/simple', data=data)

		# update pc_idt cookie
		self.headers = {'Host': 'pixelzirkus.gameforge.com', 'User-Agent': user_agent, 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Content-Type': 'application/x-www-form-urlencoded', 'Origin': 'https://lobby.ikariam.gameforge.com', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/', 'Upgrade-Insecure-Requests': '1'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		data = {'product': 'ikariam', 'server_id': '1', 'language': 'en', 'location': 'fp_eval', 'fp_eval_id': __fp_eval_id_2, 'fingerprint': '2175408712', 'fp2_config_id': '1', 'page': 'https%3A%2F%2Flobby.ikariam.gameforge.com%2F', 'referrer': '', 'fp2_value': '921af958be7cf2f76db1e448c8a5d89d', 'fp2_exec_time': '96.00'}
		r = self.s.post('https://pixelzirkus.gameforge.com/do/simple', data=data)

		# options req (not really needed)
		self.headers = {'Host': 'gameforge.com','User-Agent': user_agent,'Accept': '*/*','Accept-Language': 'en-US,en;q=0.5','Accept-Encoding': 'gzip, deflate','Access-Control-Request-Method': 'POST','Access-Control-Request-Headers': 'content-type,tnt-installation-id','Referer': 'https://lobby.ikariam.gameforge.com/es_AR/','Origin': 'https://lobby.ikariam.gameforge.com','DNT': '1','Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.options('https://gameforge.com/api/v1/auth/thin/sessions')

		# send creds
		self.headers = {'Host': 'gameforge.com','User-Agent': user_agent,'Accept': '*/*','Accept-Language': 'en-US,en;q=0.5','Accept-Encoding': 'gzip, deflate, br','Referer': 'https://lobby.ikariam.gameforge.com/es_AR/','TNT-Installation-Id': '','Content-Type': 'application/json','Origin': 'https://lobby.ikariam.gameforge.com','DNT': '1','Connection': 'keep-alive','Pragma': 'no-cache','Cache-Control': 'no-cache','TE': 'Trailers'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		data = {"identity": self.mail, "password": self.password, "locale":"es_AR", "gfLang":"ar", "platformGameId": platformGameId, "gameEnvironmentId": gameEnvironmentId, "autoGameAccountCreation": "false"}
		r = self.s.post('https://gameforge.com/api/v1/auth/thin/sessions', json=data)
		if r.status_code == 403:
			exit(_('Wrong email or password\n'))

		# get the authentication token and set the cookie
		ses_json = json.loads(r.text, strict=False)
		auth_token = ses_json['token']
		cookie_obj = requests.cookies.create_cookie(domain='.gameforge.com', name='gf-token-production', value=auth_token)
		self.s.cookies.set_cookie(cookie_obj)

		# get accounts
		self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': 'application/json', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/hub', 'Authorization': 'Bearer {}'.format(auth_token), 'DNT': '1', 'Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://lobby.ikariam.gameforge.com/api/users/me/accounts')
		accounts = json.loads(r.text, strict=False)

		# get servers
		self.headers = {'Host': 'lobby.ikariam.gameforge.com', 'User-Agent': user_agent, 'Accept': 'application/json', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/hub', 'Authorization': 'Bearer {}'.format(auth_token), 'DNT': '1', 'Connection': 'close'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		r = self.s.get('https://lobby.ikariam.gameforge.com/api/servers')
		servers = json.loads(r.text, strict=False)

		if not self.logged:

			if len([ account for account in accounts if account['blocked'] is False ]) == 1:
				self.account  = [ account for account in accounts if account['blocked'] is False ][0]
			else:
				print(_('With which account do you want to log in?\n'))

				max_name = max( [ len(account['name']) for account in accounts if account['blocked'] is False ] )
				i = 0
				for account in [ account for account in accounts if account['blocked'] is False ]:
					server = account['server']['language']
					mundo = account['server']['number']
					world = [ srv['name'] for srv in servers if srv['language'] == server and srv['number'] == mundo ][0]
					i += 1
					pad = ' ' * (max_name - len(account['name']))
					print('({:d}) {}{} [{} - {}]'.format(i, account['name'], pad, server, world))
				num = read(min=1, max=i)
				self.account  = [ account for account in accounts if account['blocked'] is False ][num - 1]
			self.username = self.account['name']
			self.servidor = self.account['server']['language']
			self.mundo    = str(self.account['server']['number'])
			self.word     = [ srv['name'] for srv in servers if srv['language'] == self.servidor and srv['number'] == int(self.mundo) ][0]
			config.infoUser = _('Server:{}').format(self.servidor)
			config.infoUser += _(', World:{}').format(self.word)
			config.infoUser += _(', Player:{}').format(self.username)
			banner()

		resp = self.s.get('https://lobby.ikariam.gameforge.com/api/users/me/loginLink?id={}&server[language]={}&server[number]={}'.format(self.account['id'], self.servidor, self.mundo)).text
		self.s.cookies.__delitem__('PHPSESSID')
		resp = json.loads(resp, strict=False)
		if 'url' not in resp:
			if retries > 0:
				return self.__login(retries-1)
			else:
				msg = 'Login Error: ' + str(resp)
				if self.padre:
					print(msg)
					exit()
				else:
					exit(msg)

		url = resp['url']
		match = re.search(r'https://s\d+-\w{2}\.ikariam\.gameforge\.com/index\.php\?', url)
		if match is None:
			exit('Error')

		self.urlBase = match.group(0)
		self.host = self.urlBase.split('//')[1].split('/index')[0]
		self.headers = {'Host': self.host, 'User-Agent': user_agent, 'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8', 'Accept-Language': 'en-US,en;q=0.5', 'Accept-Encoding': 'gzip, deflate', 'DNT': '1', 'Connection': 'close', 'Referer': 'https://lobby.ikariam.gameforge.com/es_AR/accounts', 'Upgrade-Insecure-Requests': '1'}
		self.s.headers.clear()
		self.s.headers.update(self.headers)
		html = self.s.get(url).text

		if self.__isInVacation(html):
			msg = _('The account went into vacation mode')
			if self.padre:
				print(msg)
			else:
				sendToBot(self, msg)
			os._exit(0)
		if self.__isExpired(html):
			if self.padre:
				msg = _('Login error.')
				print(msg)
				os._exit(0)
			raise Exception('No se pudo iniciar sesión')
		self.cipher = AESCipher(self.mail, self.username, self.password)
		self.__updateCookieFile(primero=True)
		self.logged = True

	def __backoff(self):
		if self.padre is False:
			time.sleep(5 * random.randint(0, 10))

	def __sessionExpired(self):
		self.__backoff()

		sessionData = self.getSessionData()

		try:
			if sessionData['num_sesiones'] > 0 and self.s.cookies['PHPSESSID'] != sessionData['cookies']['PHPSESSID']:
				self.__getCookie(sessionData)
			else:
				try:
					self.__login(3)
				except Exception:
					self.__sessionExpired()
		except KeyError:
			try:
				self.__login(3)
			except Exception:
				self.__sessionExpired()

	def __checkCookie(self):
		sessionData = self.getSessionData()

		try:
			if sessionData['num_sesiones'] > 0:
				if self.s.cookies['PHPSESSID'] != sessionData['cookies']['PHPSESSID']:
					self.__getCookie(sessionData)
			else:
				try:
					self.__login(3)
				except Exception:
					self.__sessionExpired()
		except KeyError:
			try:
				self.__login(3)
			except Exception:
				self.__sessionExpired()

	def token(self):
		"""Generates a valid actionRequest token from the session
		Parameters
		----------
		self : Session
			Session object

		Returns
		-------
		token : str
			a string representing a valid actionRequest token
		"""
		html = self.get()
		return re.search(r'actionRequest"?:\s*"(.*?)"', html).group(1)

	def get(self, url='', params={}, ignoreExpire=False, noIndex=False):
		"""Sends get request to ikariam
		Parameters
		----------
		self : Sesion
			Session object
		url : str
			this string will be appended to the end of the urlBase of the Session object. urlBase will look like: 'https://s(number)-(country).ikariam.gameforge.com/index.php?'
		params : dict
			dictionary containing key-value pairs which represent the parameteres of the get request
		ignoreExpire: bool
			if set to True it will ignore if the current session is expired and will simply return whatever response it gets. If it's set to False, it will make sure that the current session is not expired before sending the get request, if it's expired it will login again
		noIndex : bool
			if set to True it will remove 'index.php' from the end of urlBase before appending url params and sending the get request
		
		Returns
		-------
		html : str
			response from the server
		"""
		self.__checkCookie()
		if noIndex:
			url = self.urlBase.replace('index.php', '') + url
		else:
			url = self.urlBase + url
		while True:
			try:
				html = self.s.get(url, params=params).text #this isn't recursion, this get is different from the one it's in
				if ignoreExpire is False:
					assert self.__isExpired(html) is False
				return html
			except AssertionError:
				self.__sessionExpired()
			except requests.exceptions.ConnectionError:
				time.sleep(ConnectionError_wait)

	def post(self, url='', payloadPost={}, params={}, ignoreExpire=False, noIndex=False):
		"""Sends post request to ikariam
		Parameters
		----------
		self : Sesion
			Session object
		url : str
			this string will be appended to the end of the urlBase of the Session object. urlBase will look like: 'https://s(number)-(country).ikariam.gameforge.com/index.php?'
		payloadPost : dict
			dictionary containing key-value pairs which represent the payload of the post request
		params : dict
			dictionary containing key-value pairs which represent the parameteres of the post request
		ignoreExpire: bool
			if set to True it will ignore if the current session is expired and will simply return whatever response it gets. If it's set to False, it will make sure that the current session is not expired before sending the post request, if it's expired it will login again
		noIndex : bool
			if set to True it will remove 'index.php' from the end of urlBase before appending url and params and sending the post request
		
		Returns
		-------
		html : str
			response from the server
		"""
		self.__checkCookie()
		if noIndex:
			url = self.urlBase.replace('index.php', '') + url
		else:
			url = self.urlBase + url
		while True:
			try:
				html = self.s.post(url, data=payloadPost, params=params).text
				if ignoreExpire is False:
					assert self.__isExpired(html) is False
				return html
			except AssertionError:
				self.__sessionExpired()
			except requests.exceptions.ConnectionError:
				time.sleep(ConnectionError_wait)

	def login(self):
		"""This function doesn't actually log into ikariam, it only increments a number in the .ikabot file which represents the number currently running sessions
		Parameters
		----------
		self: Session
			Session object
		"""
		self.__updateCookieFile(nuevo=True)

	def logout(self):
		"""This function decrements a number in the .ikabot file representing the number of currently running sessions. If this number is 1, it will attempt to completely log out of ikariam
		Parameters
		----------
		self: Session
			Session object
		"""
		self.__updateCookieFile(salida=True)
		if self.padre is False:
			os._exit(0)

	def setSessionData(self, sessionData):
		"""Encrypts relevant session data and writes it to the .ikabot file
		Parameters
		----------
		self : Session
			Session object
		sessionData : dict
			dictionary containing relevant session data, data is written to file using AESCipher.setSessionData
		"""
		self.cipher.setSessionData(self, sessionData)

	def getSessionData(self):
		"""Gets relevant session data from the .ikabot file
		self : Session
			Session object
		"""
		return self.cipher.getSessionData(self)

def normal_get(url, params={}):
	"""Sends a get request to provided url
	Parameters
	----------
	url : str
		a string representing the url to which to send the get request
	params : dict
		a dictionary containing key-value pairs which represent the parameters of the get request
	
	Returns
	-------
	response : requests.Response
		a requests.Response object which represents the webservers response. For more information on requests.Response refer to https://requests.readthedocs.io/en/master/api/#requests.Response
	"""
	try:
		return requests.get(url, params=params)
	except requests.exceptions.ConnectionError:
		sys.exit(_('Internet connection failed'))



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import getpass
import os
import gettext
import ikabot.config as config

t = gettext.translation('gui', 
                        config.localedir, 
                        languages=config.idiomas,
                        fallback=True)
_ = t.gettext

def enter():
	"""Wait for the user to press Enter
	"""
	if config.isWindows:
		input(_('\n[Enter]')) # TODO improve this
	else:
		getpass.getpass(_('\n[Enter]'))

def clear():
	"""Clears all text on the console
	"""
	if config.isWindows:
		os.system('cls')
	else:
		os.system('clear')

def banner():
	"""Clears all text on the console and displays the Ikabot ASCII art banner
	"""
	clear()
	bner = """
	`7MMF'  `7MM                       `7MM\"""Yp,                 mm    
	  MM      MM                         MM    Yb                 MM    
	  MM      MM  ,MP'   ,6"Yb.          MM    dP    ,pW"Wq.    mmMMmm  
	  MM      MM ;Y     8)   MM          MM\"""bg.   6W'   `Wb     MM    
	  MM      MM;Mm      ,pm9MM          MM    `Y   8M     M8     MM    
	  MM      MM `Mb.   8M   MM          MM    ,9   YA.   ,A9     MM    
	.JMML.  .JMML. YA.  `Moo9^Yo.      .JMMmmmd9     `Ybmd9'      `Mbmo
	"""
	print('\n{}\n\n{}\n{}'.format(bner, config.infoUser, config.update_msg))

class bcolors:
	HEADER = '\033[95m'
	STONE = '\033[37m'
	BLUE = '\033[94m'
	GREEN = '\033[92m'
	WARNING = '\033[93m'
	RED = '\033[91m'
	BLACK = '\033[90m'
	ENDC = '\033[0m'
	BOLD = '\033[1m'
	UNDERLINE = '\033[4m'



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import gettext
import traceback
import sys
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.gui import enter
from ikabot.helpers.pedirInfo import getIdsOfCities
from ikabot.helpers.varios import wait

t = gettext.translation('loginDaily', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def loginDaily(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		print(_('I will enter every day.'))
		enter()
	except KeyboardInterrupt:
		e.set()
		return

	set_child_mode(s)
	e.set()

	info = _('\nI enter every day\n')
	setInfoSignal(s, info)
	try:
		do_it(s)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(s, msg)
	finally:
		s.logout()

def do_it(s):
	while True:
		(ids, cities) = getIdsOfCities(s)
		s.get('action=AvatarAction&function=giveDailyActivityBonus&dailyActivityBonusCitySelect={}&startPageShown=1&detectedDevice=1&autoLogin=on&cityId={}&activeTab=multiTab2&backgroundView=city&currentCityId={}&actionRequest={}&ajax=1'.format(str(ids[0]), str(ids[0]), str(ids[0]), s.token()))
		wait(24*60*60, 1*60*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.planearViajes import executeRoutes
from ikabot.helpers.recursos import *
from ikabot.helpers.varios import addDot
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.gui import banner

t = gettext.translation('distributeResources',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def distributeResources(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		banner()

		print(_('What resource do you want to distribute?'))
		print(_('(0) Exit'))
		for i in range(len(materials_names)):
			print('({:d}) {}'.format(i+1, materials_names[i]))
		resource = read(min=0, max=5)
		if resource == 0:
			e.set() #give main process control before exiting
			return
		resource -= 1

		if resource == 0:
			evenly = True
		else:
			print('\nDistributes resources from cities that do produce them \nto cities that do not (1) or distribute them evenly among all cities (2)?')
			type_distribution = read(min=1, max=2)
			evenly = type_distribution == 2

		if evenly:
			routes = distribute_evenly(s, resource)
		else:
			routes = distribute_unevenly(s, resource)

		if routes is None:
			e.set()
			return

		banner()
		print(_('\nThe following shipments will be made:\n'))
		for route in routes:
			print('{} -> {} : {} {}'.format(route[0]['name'], route[1]['name'], route[resource+3], materials_names[resource])) #displays all routes to be executed in console

		print(_('\nProceed? [Y/n]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			e.set()
			return

	except KeyboardInterrupt:
		e.set()
		return

	set_child_mode(s)
	e.set() #this is where we give back control to main process

	info = _('\nDistribute {}\n').format(materials_names[resource])
	setInfoSignal(s, info)

	try:
		executeRoutes(s, routes) #plan trips for all the routes
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(s, msg) #sends message to telegram bot
	finally:
		s.logout()

def distribute_evenly(s, resource):
	resourceTotal = 0
	(cityIDs, cities) = getIdsOfCities(s)

	originCities = {}
	destinationCities = {}
	allCities = {}
	for cityID in cityIDs:

		html = s.get(urlCiudad + cityID) #load html from the get request for that particular city
		city = getCiudad(html) #convert the html to a city object

		resourceTotal += city['recursos'][resource] #the cities resources are added to the total
		allCities[cityID] = city #adds the city to all cities


	# if a city doesn't have enough storage to fit resourceAverage
	# ikabot will send enough resources to fill the store to the max
	# then, resourceAverage will be recalculated
	resourceAverage = resourceTotal // len(allCities)
	while True:

		len_prev = len(destinationCities)
		for cityID in allCities:
			if cityID in destinationCities:
				continue
			freeStorage = allCities[cityID]['freeSpaceForResources'][resource]
			storage = allCities[cityID]['storageCapacity']
			if storage < resourceAverage:
				destinationCities[cityID] = freeStorage
				resourceTotal -= storage

		resourceAverage = resourceTotal // ( len(allCities) - len(destinationCities) )

		if len_prev == len(destinationCities):
			for cityID in allCities:
				if cityID in destinationCities:
					continue
				if allCities[cityID]['recursos'][resource] > resourceAverage:
					originCities[cityID] = allCities[cityID]['recursos'][resource] - resourceAverage
				else:
					destinationCities[cityID] = resourceAverage - allCities[cityID]['recursos'][resource]
			break

	originCities = {k: v for k, v in sorted(originCities.items(), key=lambda item: item[1],reverse=True)} #sort origin cities in descending order
	destinationCities = {k: v for k, v in sorted(destinationCities.items(), key=lambda item: item[1])}    #sort destination cities in ascending order

	routes = []

	for originCityID in originCities: #iterate through all origin city ids

		for destinationCityID in destinationCities: #iterate through all destination city ids
			if originCities[originCityID] == 0 or destinationCities[destinationCityID] == 0:
				continue

			if originCities[originCityID] > destinationCities[destinationCityID]: #if there's more resources above average in the origin city than resources below average in the destination city (origin city needs to have a surplus and destination city needs to have a deficit of resources for a route to be considered)
				toSend = destinationCities[destinationCityID] #number of resources to send is the number of resources below average in destination city
			else:
				toSend = originCities[originCityID] #send the amount of resources above average of the current origin city

			if toSend == 0:
				continue

			toSendArr = [0] * len(materials_names)
			toSendArr[resource] = toSend
			route = (allCities[originCityID], allCities[destinationCityID], allCities[destinationCityID]['islandId'], *toSendArr)
			routes.append(route)

			# ROUTE BLOCK
			if originCities[originCityID] > destinationCities[destinationCityID]:
				originCities[originCityID] -= destinationCities[destinationCityID] #remove the sent amount from the origin city's surplus
				destinationCities[destinationCityID] = 0 #set the amount of resources below average in destination city to 0
			else:
				destinationCities[destinationCityID] -= originCities[originCityID] #remove the sent amount from the amount of resources below average in current destination city
				originCities[originCityID] = 0 #set the amount of resources above average in origin city to 0

	return routes

def distribute_unevenly(s, recurso):

	recursoTotal = 0
	(idsCiudades, ciudades) = getIdsOfCities(s)
	ciudadesOrigen = {}
	ciudadesDestino = {}
	for idCiudad in idsCiudades:
		esTarget =  ciudades[idCiudad]['tradegood'] == str(recurso)
		if esTarget:
			html = s.get(urlCiudad + idCiudad)
			ciudad = getCiudad(html)
			if recurso == 1:
				ciudad['disponible'] = ciudad['recursos'][recurso] - ciudad['consumo'] - 1
			else:
				ciudad['disponible'] = ciudad['recursos'][recurso]
			if ciudad['disponible'] < 0:
				ciudad['disponible'] = 0
			recursoTotal += ciudad['disponible']
			ciudadesOrigen[idCiudad] = ciudad
		else:
			html = s.get(urlCiudad + idCiudad)
			ciudad = getCiudad(html)
			ciudad['disponible'] = ciudad['freeSpaceForResources'][recurso]
			if ciudad['disponible'] > 0:
				ciudadesDestino[idCiudad] = ciudad

	if recursoTotal <= 0:
		print(_('\nThere are no resources to send.'))
		enter()
		return None
	if len(ciudadesDestino) == 0:
		print(_('\nThere is no space available to send resources.'))
		enter()
		return None

	recursoXciudad = recursoTotal // len(ciudadesDestino)
	espacios_disponibles = [ ciudadesDestino[city]['disponible'] for city in ciudadesDestino ]
	totalEspacioDisponible = sum( espacios_disponibles )
	restanteAEnviar = min(recursoTotal, totalEspacioDisponible)
	toSend = {}

	while restanteAEnviar > 0:
		len_prev = len(toSend)
		for city in ciudadesDestino:
			ciudad = ciudadesDestino[city]
			if city not in toSend and ciudad['disponible'] < recursoXciudad:
				toSend[city] = ciudad['disponible']
				restanteAEnviar -= ciudad['disponible']

		if len(toSend) == len_prev:
			for city in ciudadesDestino:
				if city not in toSend:
					toSend[city] = recursoXciudad
			break

		espacios_disponibles = [ ciudadesDestino[city]['disponible'] for city in ciudadesDestino if city not in toSend ]
		totalEspacioDisponible = sum( espacios_disponibles )
		restanteAEnviar = min(restanteAEnviar, totalEspacioDisponible)
		recursoXciudad = restanteAEnviar // len(espacios_disponibles)


	rutas = []
	for idCiudad in ciudadesDestino:
		ciudadD = ciudadesDestino[idCiudad]
		idIsla = ciudadD['islandId']
		faltante = toSend[idCiudad]
		for idCiudadOrigen in ciudadesOrigen:
			if faltante == 0:
				break

			ciudadO = ciudadesOrigen[idCiudadOrigen]
			recursoDisponible = ciudadO['disponible']
			for ruta in rutas:
				origen = ruta[0]
				rec = ruta[recurso + 3]
				if origen['id'] == idCiudadOrigen:
					recursoDisponible -= rec

			enviar = min(faltante, recursoDisponible)
			disponible = ciudadD['disponible']
			if disponible == 0 or enviar == 0:
				continue

			if disponible < enviar:
				faltante = 0
				enviar = disponible
			else:
				faltante -= enviar

			toSendArr = [0] * len(materials_names)
			toSendArr[recurso] = enviar
			ruta = (ciudadO, ciudadD, idIsla, *toSendArr)

			rutas.append(ruta)

	return rutas



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import gettext
import traceback
import sys
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.gui import enter
from ikabot.helpers.varios import wait
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.pedirInfo import getIdsOfIslands
from ikabot.helpers.getJson import getIsla
from ikabot.helpers.process import set_child_mode

t = gettext.translation('searchForIslandSpaces', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def searchForIslandSpaces(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		if botValido(s) is False:
			e.set()
			return
		print(_('I will search for new spaces each hour.'))
		enter()
	except KeyboardInterrupt:
		e.set()
		return

	set_child_mode(s)
	e.set()

	info = _('\nI search for new spaces each hour\n')
	setInfoSignal(s, info)
	try:
		do_it(s)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(s, msg)
	finally:
		s.logout()

def do_it(s):
	isla_ciudades = {}
	while True:
		idIslas = getIdsOfIslands(s) #gets the ids of the islands
		for idIsla in idIslas: #for each island id
			html = s.get(urlIsla + idIsla) #get html
			isla = getIsla(html) #parse html into island object
			ciudades = [ciudad for ciudad in isla['cities'] if ciudad['type'] != 'empty'] #loads the islands non empty cities into ciudades

			if idIsla in isla_ciudades: #for each island
				ciudadesAntes = isla_ciudades[idIsla] #loads into ciudadesAntes the current islands cities

				# alguien desaparecio - someone disappeared
				for cityAntes in ciudadesAntes: #for each beforecity on the island
					for ciudad in ciudades: #for each city
						if ciudad['id'] == cityAntes['id']: #compare current city's id with beforecity's id
							break
					else:
						msg = _('the city {} of the player {} disappeared in {} {}:{} {}').format(cityAntes['name'], cityAntes['Name'], materials_names[int(isla['good'])], isla['x'], isla['y'], isla['name'])
						sendToBot(s, msg)

				# alguien fundo - someone colonised
				for ciudad in ciudades: #for each city on the island
					for cityAntes in ciudadesAntes: #for each beforecity
						if ciudad['id'] == cityAntes['id']: #compare current city's id with beforecity's id
							break
					else:
						msg = _('{} founded {} in {} {}:{} {}').format(ciudad['Name'], ciudad['name'], materials_names[int(isla['good'])], isla['x'], isla['y'], isla['name'])
						sendToBot(s, msg)

			isla_ciudades[idIsla] = ciudades.copy() #copies non empty cities into current islands cities (isla_ciudades)
		wait(1*60*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import os
import json
import random
import gettext
import sys
import ikabot.web.sesion
from ikabot.config import *
import ikabot.config as config
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import read

t = gettext.translation('botComm', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def sendToBotDebug(s ,msg, debugON):
	if debugON:
		sendToBot(s, msg)

def sendToBot(s, msg, Token=False):
	if Token is False:
		msg = 'pid:{}\n{}\n{}'.format(os.getpid(), config.infoUser, msg)
	sessionData = s.getSessionData()
	try:
		ikabot.web.sesion.normal_get('https://api.telegram.org/bot{}/sendMessage'.format(sessionData['telegram']['botToken']), params={'chat_id': sessionData['telegram']['chatId'], 'text': msg})
	except KeyError:
		pass

def telegramCredsValidas(s):
	sessionData = s.getSessionData()
	try:
		return len(sessionData['telegram']['botToken']) > 0 and len(sessionData['telegram']['chatId']) > 0
	except KeyError:
		return False

def getUserResponse(s):
	# returns messages that the user sends to the telegram bot
	sessionData = s.getSessionData()
	try:
		updates = ikabot.web.sesion.normal_get('https://api.telegram.org/bot{}/getUpdates'.format(sessionData['telegram']['botToken'])).text
		updates = json.loads(updates, strict=False)
		if updates['ok'] is False:
			return []
		updates = updates['result']
		# only return messages from the chatId of our user
		return [update['message']['text'] for update in updates if update['message']['chat']['id'] == int(sessionData['telegram']['chatId'])]
	except KeyError:
		return []

def botValido(s):
	if telegramCredsValidas(s):
		return True
	else:
		banner()
		print(_('You must provide valid credentials to communicate by telegram.'))
		print(_('You require the token of the bot you are going to use and your chat_id'))
		print(_('For more information about how to obtain them read the readme at https://github.com/physics-sp/ikabot'))
		rta = read(msg=_('Will you provide the credentials now? [y/N]'), values=['y','Y','n', 'N', ''])
		if rta.lower() != 'y':
			return False
		else:
			return updateTelegramData(s)

def updateTelegramData(s, e=None, fd=None):
	if e is not None and fd is not None:
		sys.stdin = os.fdopen(fd) # give process access to terminal

	banner()
	botToken = read(msg=_('Bot\'s token:'))
	chat_id = read(msg=_('Chat_id:'))

	sessionData = s.getSessionData()
	sessionData['telegram'] = {}
	sessionData['telegram']['botToken'] = botToken.replace(' ', '').replace('.', '')
	sessionData['telegram']['chatId'] = chat_id
	s.setSessionData(sessionData)

	rand = random.randint(1000, 9999)
	msg = _('El token a ingresar es:{:d}').format(rand)
	sendToBot(s, msg, Token=True)

	rta = read(msg=_('A message was sent by telegram, did you receive it? [Y/n]'), values=['y','Y','n', 'N', ''])
	if rta.lower() == 'n':
		valid = False
	else:
		recibido = read(msg=_('Enter the received token in telegram:'), digit=True)
		if rand != recibido:
			print(_('The token is incorrect'))
			valid = False
		else:
			print(_('The token is correct'))
			valid = True

	if valid is False:
		sessionData['telegram']['botToken'] = ''
		sessionData['telegram']['chatId'] = ''
		s.setSessionData(sessionData)
		print(_('Check the credentials and re-supply them.'))
	else:
		print(_('The data was saved.'))
	enter()

	if e is not None and fd is not None:
		e.set() #give main process control before exiting
	return valid



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import time
import math
import json
from decimal import *
from ikabot.config import *
from ikabot.helpers.varios import wait
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.naval import *

def sendGoods(s, originCityId, destinationCityId, islandId, ships, send):
	"""This function will execute one route
	Parameters
	----------
	s : Session
		Session object
	originCityId : int
		integer representing the ID of the origin city
	destinationCityId : int
		integer representing the ID of the destination city
	islandId : int
		integer representing the ID of the destination city's island
	ships : int
		integer representing the amount of ships needed to execute the route
	send : array
		array of resources to send
	"""

	html = s.get()
	city = getCiudad(html)
	currId = city['id']
	data = {'action': 'header', 'function': 'changeCurrentCity', 'actionRequest': s.token(), 'oldView': 'city', 'cityId': originCityId, 'backgroundView': 'city', 'currentCityId': currId, 'ajax': '1'}
	s.post(payloadPost=data)

	data = {'action': 'transportOperations', 'function': 'loadTransportersWithFreight', 'destinationCityId': destinationCityId, 'islandId': islandId, 'oldView': '', 'position': '', 'avatar2Name': '', 'city2Name': '', 'type': '', 'activeTab': '', 'transportDisplayPrice': '0', 'premiumTransporter': '0', 'minusPlusValue': '500', 'capacity': '5', 'max_capacity': '5', 'jetPropulsion': '0', 'transporters': ships, 'backgroundView': 'city', 'currentCityId': originCityId, 'templateView': 'transport', 'currentTab': 'tabSendTransporter', 'actionRequest': s.token(), 'ajax': '1'}

	# add amounts of resources to send
	for i in range(len(send)):
		key = 'cargo_resource' if i == 0 else 'cargo_tradegood{:d}'.format(i)
		data[key] = send[i]

	s.post(payloadPost=data)

def executeRoutes(s, routes):
	"""This function will execute all the routes passed to it, regardless if there are enough ships available to do so
	Parameters
	----------
	s : Session
		Session object
	routes : list
		a list of tuples, each of which represent a route. A route is defined like so : (originCity,destinationCity,islandId,wood,wine,marble,crystal,sulfur). originCity and destintionCity should be passed as City objects 
	"""
	for ruta in routes:
		(ciudadOrigen, ciudadDestino, idIsla, *toSend) = ruta
		destId = ciudadDestino['id']

		while sum(toSend) > 0:
			barcosDisp = waitForArrival(s)
			storageCapacityInShips = barcosDisp * 500

			html = s.get(urlCiudad + destId)
			ciudadDestino = getCiudad(html)
			storageCapacityInCity = ciudadDestino['freeSpaceForResources']

			send = []
			for i in range(len(toSend)):
				min_val = min(toSend[i], storageCapacityInShips, storageCapacityInCity[i])
				send.append(min_val)
				storageCapacityInShips -= send[i]
				toSend[i] -= send[i]

			cantEnviada = sum(send)
			if cantEnviada == 0:
				# no space available
				# wait an hour and try again
				wait(60 * 60)
				continue

			barcos = int(math.ceil((Decimal(cantEnviada) / Decimal(500))))
			sendGoods(s, ciudadOrigen['id'], ciudadDestino['id'], idIsla, barcos, send)

def getMinimumWaitingTime(s):
	"""This function returns the time needed to wait for the closest fleet to arrive. If all ships are unavailable, this represents the minimum time needed to wait for any ships to become available
	Parameters
	----------
	s : Session
		Session object

	Returns
	-------
	timeToWait : int
		the minimum waiting time for the closest fleet to arrive 
	"""
	html = s.get()
	idCiudad = re.search(r'currentCityId:\s(\d+),', html).group(1)
	url = 'view=militaryAdvisor&oldView=city&oldBackgroundView=city&backgroundView=city&currentCityId={}&actionRequest={}&ajax=1'.format(idCiudad, s.token())
	posted = s.post(url)
	postdata = json.loads(posted, strict=False)
	militaryMovements = postdata[1][1][2]['viewScriptParams']['militaryAndFleetMovements']
	tiempoAhora = int(postdata[0][1]['time'])
	tiemposDeEspera = []
	for militaryMovement in [ mv for mv in militaryMovements if mv['isOwnArmyOrFleet'] ]:
		tiempoRestante = int(militaryMovement['eventTime']) - tiempoAhora
		tiemposDeEspera.append(tiempoRestante)
	if tiemposDeEspera:
		return min(tiemposDeEspera)
	else:
		return 0

def waitForArrival(s):
	"""This function will return the number of available ships, and if there aren't any, it will wait for the closest fleet to arrive and then return the number of available ships
	Parameters
	----------
	s : Session
		Session object

	Returns
	-------
	ships : int
		number of available ships
	"""
	barcos = getAvailableShips(s)
	while barcos == 0:
		minTiempoDeEspera = getMinimumWaitingTime(s)
		wait( minTiempoDeEspera )
		barcos = getAvailableShips(s)
	return barcos



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import math
import json
import gettext
import traceback
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.tienda import *
from ikabot.helpers.botComm import *
from ikabot.helpers.varios import addDot
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.planearViajes import waitForArrival

t = gettext.translation('sellResources', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def elegirCiudadComercial(ciudades_comerciales):
	print(_('In which city do you want to sell resources?\n'))
	for i, ciudad in enumerate(ciudades_comerciales):
		print('({:d}) {}'.format(i + 1, ciudad['name']))
	ind = read(min=1, max=len(ciudades_comerciales))
	return ciudades_comerciales[ind - 1]

def getStoreInfo(s, ciudad):
	params = {'view': 'branchOfficeOwnOffers', 'activeTab': 'tab_branchOfficeOwnOffers', 'cityId': ciudad['id'], 'position': ciudad['pos'], 'backgroundView': 'city', 'currentCityId': ciudad['id'], 'templateView': 'branchOfficeOwnOffers', 'currentTab': 'tab_branchOfficeOwnOffers', 'actionRequest': s.token(), 'ajax': '1'}
	resp = s.post(params=params, noIndex=True)
	return json.loads(resp, strict=False)[1][1][1]


def getOfertas(s, ciudad, recurso):
	if recurso == 0:
		recurso = 'resource'
	else:
		recurso = str(recurso)
	data = {'cityId': ciudad['id'], 'position': ciudad['pos'], 'view': 'branchOffice', 'activeTab': 'bargain', 'type': '333', 'searchResource': recurso, 'range': ciudad['rango'], 'backgroundView': 'city', 'currentCityId': ciudad['id'], 'templateView': 'branchOffice', 'currentTab': 'bargain', 'actionRequest': s.token(), 'ajax': '1'}
	resp = s.post(payloadPost=data)
	html = json.loads(resp, strict=False)[1][1][1]
	return re.findall(r'<td class=".*?">(.*?)<br/>\((.*?)\)\s*</td>\s*<td>(.*?)</td>\s*<td><img src=".*?"\s*alt=".*?"\s*title=".*?"/></td>\s*<td style="white-space:nowrap;">(\d+)\s*<img src=".*?"\s*class=".*?"/>.*?</td>\s*<td>(\d+)</td>\s*<td><a onclick="ajaxHandlerCall\(this\.href\);return false;"\s*href="\?view=takeOffer&destinationCityId=(\d+)&', html)

def venderAOfertas(s, ciudad, recurso, e):
	banner()

	matches = getOfertas(s, ciudad, recurso)

	if len(matches) == 0:
		print(_('No offers available.'))
		enter()
		return

	print(_('Which offers do you want to sell to?\n'))

	ofertas = []
	max_venta = 0
	profit    = 0
	for match in matches:
		city, user, cant, precio, dist, idDestino = match
		city = city.strip()
		cantidad = cant.replace(',', '').replace('.', '')
		cantidad = int(cantidad)
		msg = _('{} ({}): {} at {} each ({} in total) [Y/n]').format(city, user, addDot(cantidad), precio, addDot(int(precio)*cantidad))
		rta = read(msg=msg, values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			continue
		ofertas.append(match)
		max_venta += cantidad
		profit += cantidad * int(precio)
	banner()
	if len(ofertas) == 0:
		return

	disp_venta = ciudad['recursos'][recurso]
	vender = disp_venta if disp_venta < max_venta else max_venta

	print(_('\nHow much do you want to sell? [max = {}]').format(addDot(vender)))
	vender = read(min=0, max=vender)

	faltaVender = vender
	profit    = 0
	for oferta in ofertas:
		city, user, cant, precio, dist, idDestino = oferta
		city = city.strip()
		cantidad = cant.replace(',', '').replace('.', '')
		cantidad = int(cantidad)
		compra = cantidad if cantidad < faltaVender else faltaVender
		faltaVender -= compra
		profit += compra * int(precio)
	print(_('\nSell {} of {} for a total of {}? [Y/n]').format(addDot(vender), materials_names[recurso], addDot(profit)))
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		return

	set_child_mode(s)
	e.set()

	info = _('\nI sell {} of {} in {}\n').format(addDot(vender), materials_names[recurso], ciudad['name'])
	setInfoSignal(s, info)
	try:
		do_it1(s, vender,  ofertas, recurso, ciudad)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(s, msg)
	finally:
		s.logout()

def crearOferta(s, ciudad, recurso, e):
	banner()

	html = getStoreInfo(s, ciudad)
	cap_venta = getstorageCapacityDeVenta(html)
	recurso_disp = ciudad['recursos'][recurso]
	print(_('How much do you want to sell? [max = {}]').format(addDot(recurso_disp)))
	vender = read(min=0, max=recurso_disp)
	if vender == 0:
		return

	precio_max, precio_min = re.findall(r'\'upper\': (\d+),\s*\'lower\': (\d+)', html)[recurso]
	precio_max = int(precio_max)
	precio_min = int(precio_min)
	print(_('\nAt what price? [min = {:d}, max = {:d}]').format(precio_min, precio_max))
	precio = read(min=precio_min, max=precio_max)

	print(_('\nI will sell {} of {} at {}: {}').format(addDot(vender), materials_names[recurso], addDot(precio), addDot(precio * vender)))
	print(_('\nProceed? [Y/n]'))
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		return

	set_child_mode(s)
	e.set()

	info = _('\nI sell {} of {} in {}\n').format(addDot(vender), materials_names[recurso], ciudad['name'])
	setInfoSignal(s, info)
	try:
		do_it2(s, vender, precio, recurso, cap_venta, ciudad)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(s, msg)
	finally:
		s.logout()

def sellResources(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		banner()

		ciudades_comerciales = getCommertialCities(s)
		if len(ciudades_comerciales) == 0:
			print(_('There is no shop built'))
			enter()
			e.set()
			return

		if len(ciudades_comerciales) == 1:
			ciudad = ciudades_comerciales[0]
		else:
			ciudad = elegirCiudadComercial(ciudades_comerciales)
			banner()

		print(_('What resource do you want to sell?'))
		for indice, bien in enumerate(materials_names):
			print('({:d}) {}'.format(indice+1, bien))
		eleccion = read(min=1, max=len(materials_names))
		recurso = eleccion - 1
		banner()

		print(_('Do you want to sell to existing offers (1) or do you want to make your own offer (2)?'))
		rta = read(min=1, max=2)
		[venderAOfertas, crearOferta][rta - 1](s, ciudad, recurso, e)
		e.set()
	except KeyboardInterrupt:
		e.set()
		return

def do_it1(s, porVender, ofertas, recurso, ciudad):
	for oferta in ofertas:
		city, user, cant, precio, dist, idDestino = oferta
		city = city.strip()
		quiereComprar = cant.replace(',', '').replace('.', '')
		quiereComprar = int(quiereComprar)
		while True:
			barcos_disponibles = waitForArrival(s)
			cant_venta = quiereComprar if quiereComprar < porVender else porVender
			barcos_necesarios = int(math.ceil((Decimal(cant_venta) / Decimal(500))))
			barcos_usados = barcos_disponibles if barcos_disponibles < barcos_necesarios else barcos_necesarios
			if barcos_necesarios > barcos_usados:
				cant_venta = barcos_usados * 500
			porVender -= cant_venta
			quiereComprar -= cant_venta

			data = {'action': 'transportOperations', 'function': 'sellGoodsAtAnotherBranchOffice', 'cityId': ciudad['id'], 'destinationCityId': idDestino, 'oldView': 'branchOffice', 'position': ciudad['pos'], 'avatar2Name': user, 'city2Name': city, 'type': '333', 'activeTab': 'bargain', 'transportDisplayPrice': '0', 'premiumTransporter': '0', 'capacity': '5', 'max_capacity': '5', 'jetPropulsion': '0', 'transporters': str(barcos_usados), 'backgroundView': 'city', 'currentCityId': ciudad['id'], 'templateView': 'takeOffer', 'currentTab': 'bargain', 'actionRequest': s.token(), 'ajax': '1'}
			if recurso == 0:
				data['cargo_resource'] = str(cant_venta)
				data['resourcePrice'] = str(precio)
			else:
				data['tradegood{:d}Price'.format(recurso)] = str(precio)
				data['cargo_tradegood{:d}'.format(recurso)] = str(cant_venta)
			s.get('view=city&cityId={}'.format(ciudad['id']), noIndex=True)
			s.post(payloadPost=data)

			if porVender == 0:
				return
			if quiereComprar == 0:
				break

def do_it2(s, porVender, precio, recurso, cap_venta, ciudad):
	total = porVender
	html = getStoreInfo(s, ciudad)
	enVenta_inicial = vendiendo(html)[recurso]
	while True:
		html = getStoreInfo(s, ciudad)
		enVenta = vendiendo(html)[recurso]
		if enVenta < getstorageCapacityDeVenta(html):
			espacio = cap_venta - enVenta
			ofertar = porVender if espacio > porVender else espacio
			porVender -= ofertar
			nuevaVenta = enVenta + ofertar

			payloadPost = {'cityId': ciudad['id'], 'position': ciudad['pos'], 'action': 'CityScreen', 'function': 'updateOffers', 'resourceTradeType': '444', 'resource': '0', 'resourcePrice': '10', 'tradegood1TradeType': '444', 'tradegood1': '0', 'tradegood1Price': '11', 'tradegood2TradeType': '444', 'tradegood2': '0', 'tradegood2Price': '12', 'tradegood3TradeType': '444', 'tradegood3': '0', 'tradegood3Price': '17', 'tradegood4TradeType': '444', 'tradegood4': '0', 'tradegood4Price': '5', 'backgroundView': 'city', 'currentCityId': ciudad['id'], 'templateView': 'branchOfficeOwnOffers', 'currentTab': 'tab_branchOfficeOwnOffers', 'actionRequest': s.token(), 'ajax': '1'}
			if recurso == 0:
				payloadPost['resource'] = str(nuevaVenta)
				payloadPost['resourcePrice'] = str(precio)
			else:
				payloadPost['tradegood{:d}'.format(recurso)] = str(nuevaVenta)
				payloadPost['tradegood{:d}Price'.format(recurso)] = str(precio)
			s.post(payloadPost=payloadPost)

			if porVender == 0:
				break
		time.sleep(60 * 60 *  2)

	while True:
		html = getStoreInfo(s, ciudad)
		enVenta = vendiendo(html)[recurso]
		if enVenta <= enVenta_inicial:
			msg = _('{} of {} was sold at {:d}').format(addDot(total), materials_names[recurso], precio)
			sendToBot(s, msg)
			return
		time.sleep(60 * 60 *  2)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import math
import json
import gettext
import traceback
from decimal import *
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.varios import addDot
from ikabot.helpers.gui import enter, banner
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.planearViajes import waitForArrival
from ikabot.helpers.pedirInfo import getIdsOfCities, read
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.recursos import *
from ikabot.helpers.tienda import *

t = gettext.translation('buyResources', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def chooseResource(s, city):
	print(_('Which resource do you want to buy?'))
	for index, material_name in enumerate(materials_names):
		print('({:d}) {}'.format(index+1, material_name))
	choise = read(min=1, max=5)
	resource = choise - 1
	if resource == 0:
		resource = 'resource'
	data = {
	'cityId': city['id'],
	'position': city['pos'],
	'view': 'branchOffice',
	'activeTab': 'bargain',
	'type': 444,
	'searchResource': resource,
	'range': city['rango'],
	'backgroundView' : 'city',
	'currentCityId': city['id'],
	'templateView': 'branchOffice',
	'currentTab': 'bargain',
	'actionRequest': s.token(),
	'ajax': 1
	}
	# this will set the chosen resource in the store
	s.post(payloadPost=data)
	resource = choise - 1
	# return the chosen resource
	return resource

def getOffers(s, city):
	html = getStoreHtml(s, city)
	hits = re.findall(r'short_text80">(.*?) *<br/>\((.*?)\)\s *</td>\s *<td>(\d+)</td>\s *<td>(.*?)/td>\s *<td><img src="skin/resources/icon_(\w+)\.png[\s\S]*?white-space:nowrap;">(\d+)\s[\s\S]*?href="\?view=takeOffer&destinationCityId=(\d+)&oldView=branchOffice&activeTab=bargain&cityId=(\d+)&position=(\d+)&type=(\d+)&resource=(\w+)"', html)
	offers = []
	for hit in hits:
		offer = {
		'ciudadDestino': hit[0],
		'jugadorAComprar' : hit[1],
		'bienesXminuto': int(hit[2]),
		'amountAvailable': int(hit[3].replace(',', '').replace('.', '').replace('<', '')),
		'tipo': hit[4],
		'precio': int(hit[5]),
		'destinationCityId': hit[6],
		'cityId': hit[7],
		'position': hit[8],
		'type': hit[9],
		'resource': hit[10]
		}
		offers.append(offer)
	return offers

def calculateCost(offers, amount_to_buy):
	total_cost = 0
	for offer in offers:
		if amount_to_buy == 0:
			break
		buy = min(offer['amountAvailable'], amount_to_buy)
		amount_to_buy -= buy
		total_cost += buy * offer['precio']
	return total_cost

def getGold(s, ciudad):
	url = 'view=finances&backgroundView=city&currentCityId={}&templateView=finances&actionRequest={}&ajax=1'.format(ciudad['id'], s.token())
	data = s.post(url)
	json_data = json.loads(data, strict=False)
	gold = json_data[0][1]['headerData']['gold']
	gold = gold.split('.')[0]
	gold = int(gold)
	return gold

def chooseCommertialCity(commercial_cities):
	print(_('From which city do you want to buy resources?\n'))
	for i, ciudad in enumerate(commercial_cities):
		print('({:d}) {}'.format(i + 1, ciudad['name']))
	ind = read(min=1, max=len(commercial_cities))
	return commercial_cities[ind - 1]

def buyResources(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		banner()

		# get all the cities with a store
		commercial_cities = getCommertialCities(s)
		if len(commercial_cities) == 0:
			print(_('There is no store build'))
			enter()
			e.set()
			return

		# choose which city to buy from
		if len(commercial_cities) == 1:
			city = commercial_cities[0]
		else:
			city = chooseCommertialCity(commercial_cities)
			banner()

		# choose resource to buy
		resource = chooseResource(s, city)
		banner()

		# get all the offers of the chosen resource from the chosen city
		offers = getOffers(s, city)
		if len(offers) == 0:
			print(_('There are no offers available.'))
			e.set()
			return

		# display offers to the user
		total_price   = 0
		total_amount = 0
		for offer in offers:
			amount = offer['amountAvailable']
			price  = offer['precio']
			cost   = amount * price
			print(_('amount:{}').format(addDot(amount)))
			print(_('price :{:d}').format(price))
			print(_('cost  :{}').format(addDot(cost)))
			print('')
			total_price += cost
			total_amount += amount

		# ask how much to buy
		print(_('Total amount available to purchase: {}, for {}').format(addDot(total_amount), addDot(total_price)))
		available = city['freeSpaceForResources'][resource]
		if available < total_amount:
			print(_('You just can buy {} due to storing capacity').format(addDot(available)))
			total_amount = available
		print('')
		amount_to_buy = read(msg=_('How much do you want to buy?: '), min=0, max=total_amount)
		if amount_to_buy == 0:
			e.set()
			return

		# calculate the total cost
		gold = getGold(s, city)
		total_cost = calculateCost(offers, amount_to_buy)

		print(_('\nCurrent gold: {}.\nTotal cost  : {}.\nFinal gold  : {}.'). format(addDot(gold), addDot(total_cost), addDot(gold - total_cost)))
		print(_('Proceed? [Y/n]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			e.set()
			return

		print(_('It will be purchased {}').format(addDot(amount_to_buy)))
		enter()
	except KeyboardInterrupt:
		e.set()
		return

	set_child_mode(s)
	e.set()

	info = _('\nI will buy {} from {} to {}\n').format(addDot(amount_to_buy), materials_names[resource], city['cityName'])
	setInfoSignal(s, info)
	try:
		do_it(s, city, offers, amount_to_buy)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(s, msg)
	finally:
		s.logout()

def buy(s, city, offer, amount_to_buy):
	ships = int(math.ceil((Decimal(amount_to_buy) / Decimal(500))))
	data_dict = {
	'action': 'transportOperations',
	'function': 'buyGoodsAtAnotherBranchOffice',
	'cityId': offer['cityId'],
	'destinationCityId': offer['destinationCityId'],
	'oldView': 'branchOffice',
	'position': city['pos'],
	'avatar2Name': offer['jugadorAComprar'],
	'city2Name': offer['ciudadDestino'],
	'type': int(offer['type']),
	'activeTab': 'bargain',
	'transportDisplayPrice': 0,
	'premiumTransporter': 0,
	'capacity': 5,
	'max_capacity': 5,
	'jetPropulsion': 0,
	'transporters': ships,
	'backgroundView': 'city',
	'currentCityId': offer['cityId'],
	'templateView': 'takeOffer',
	'currentTab': 'bargain',
	'actionRequest': s.token(),
	'ajax': 1
	}
	url = 'view=takeOffer&destinationCityId={}&oldView=branchOffice&activeTab=bargain&cityId={}&position={}&type={}&resource={}&backgroundView=city&currentCityId={}&templateView=branchOffice&actionRequest={}&ajax=1'.format(offer['destinationCityId'], offer['cityId'], offer['position'], offer['type'], offer['resource'], offer['cityId'], s.token())
	data = s.post(url)
	html = json.loads(data, strict=False)[1][1][1]
	hits = re.findall(r'"tradegood(\d)Price"\s*value="(\d+)', html)
	for hit in hits:
		data_dict['tradegood{}Price'.format(hit[0])] = int(hit[1])
		data_dict['cargo_tradegood{}'.format(hit[0])] = 0
	hit = re.search(r'"resourcePrice"\s*value="(\d+)', html)
	if hit:
		data_dict['resourcePrice'] = int(hit.group(1))
		data_dict['cargo_resource'] = 0
	resource = offer['resource']
	if resource == 'resource':
		data_dict['cargo_resource'] = amount_to_buy
	else:
		data_dict['cargo_tradegood{}'.format(resource)] = amount_to_buy
	s.post(payloadPost=data_dict)
	msg = _('I buy {} to {} from {}').format(addDot(amount_to_buy), offer['ciudadDestino'], offer['jugadorAComprar'])
	sendToBotDebug(s, msg, debugON_buyResources)

def do_it(s, city, offers, amount_to_buy):
	while True:
		for offer in offers:
			if amount_to_buy == 0:
				return
			if offer['amountAvailable'] == 0:
				continue

			ships_available = waitForArrival(s)
			storageCapacity  = ships_available * 500
			buy_amount = min(amount_to_buy, storageCapacity, offer['amountAvailable'])

			amount_to_buy -= buy_amount
			offer['amountAvailable'] -= buy_amount
			buy(s, city, offer, buy_amount)
			# start from the beginning again, so that we always buy from the cheapest offers fisrt
			break



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import sys
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.getJson import getCiudad

t = gettext.translation('vacationMode',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def activarvacationMode(s):
	html = s.get()
	ciudad = getCiudad(html)

	data = {'action': 'Options', 'function': 'activateVacationMode', 'actionRequest': s.token(), 'backgroundView': 'city', 'currentCityId': ciudad['id'], 'templateView': 'options_umod_confirm'}
	s.post(params=data, ignoreExpire=True)

def vacationMode(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		banner()
		print(_('Activate vacation mode? [Y/n]'))
		rta = read(values=['y', 'Y', 'n', 'N', ''])
		if rta.lower() == 'n':
			e.set()
			return

		activarvacationMode(s)

		print(_('Vacation mode has been activated.'))
		enter()
		e.set()
		clear()
		exit()
	except KeyboardInterrupt:
		e.set()
		return



import os
import random
import locale
import gettext

local = locale.setlocale(locale.LC_ALL, '')
if 'es_' in local:
	idiomas = ['es']
else:
	idiomas = ['en']
idiomas = ['none']

localedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'locale')
t = gettext.translation('config', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

# only use common browsers
if random.randint(0, 1) == 0:
    user_agent = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36'
else:
    user_agent = 'Mozilla/5.0 (X11; Linux x86_64; rv:77.0) Gecko/20100101 Firefox/77.0'

update_msg = ''

isWindows = os.name == 'nt'

proxy = False
if proxy:
    http_proxy  = "http://127.0.0.1:8080"
    https_proxy = "https://127.0.0.1:8080"
    proxyDict = {"http": http_proxy, "https": https_proxy}
else:
    proxyDict = {}

ids = None
ciudades = None
menuCiudades = ''
infoUser = ''
ikaFile = '.ikabot'
urlCiudad = 'view=city&cityId='
urlIsla = 'view=island&islandId='
prompt = ' >>  '
materials_names = [_('Wood'), _('Wine'), _('Marble'), _('Cristal'), _('Sulfur')]
materials_names_english = ['Wood', 'Wine', 'Marble', 'Cristal', 'Sulfur']
ConnectionError_wait = 5 * 60

debugON_alertAttacks    = False
debugON_alertLowWine   = False
debugON_donationBot        = False
debugON_searchForIslandSpaces    = False
debugON_loginDaily = False
debugON_enviarVino        = False
debugON_sendResources = False
debugON_constructionList     = False
debugON_session           = False
debugON_buyResources   = False
debugON_activateMiracle    = False



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import math
import json
import gettext
import sys
from ikabot.helpers.pedirInfo import read
from decimal import *
from ikabot.config import *
from ikabot.helpers.gui import *
from ikabot.helpers.naval import *
from ikabot.helpers.varios import *

t = gettext.translation('shipMovements',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def esHostil(movement):
	if movement['army']['amount']:
		return True
	for mov in movement['fleet']['ships']:
		if mov['cssClass'] != 'ship_transport':
			return True
	return False

def shipMovements(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		banner()

		print(_('Ships {:d}/{:d}\n').format(getAvailableShips(s), getTotalShips(s)))

		html = s.get()
		idCiudad = re.search(r'currentCityId:\s(\d+),', html).group(1)
		url = 'view=militaryAdvisor&oldView=city&oldBackgroundView=city&backgroundView=city&currentCityId={}&actionRequest={}&ajax=1'.format(idCiudad, s.token())
		posted = s.post(url)
		postdata = json.loads(posted, strict=False)
		movements = postdata[1][1][2]['viewScriptParams']['militaryAndFleetMovements']
		tiempoAhora = int(postdata[0][1]['time'])

		for movement in movements:

			color = ''
			if movement['isHostile']:
				color = bcolors.RED + bcolors.BOLD
			elif movement['isOwnArmyOrFleet']:
				color = bcolors.BLUE + bcolors.BOLD
			elif movement['isSameAlliance']:
				color = bcolors.GREEN + bcolors.BOLD

			origen  = '{} ({})'.format(movement['origin']['name'], movement['origin']['avatarName'])
			destino = '{} ({})'.format(movement['target']['name'], movement['target']['avatarName'])
			flecha = '<-' if movement['event']['isFleetReturning'] else '->'
			tiempoFaltante = int(movement['eventTime']) - tiempoAhora
			print('{}{} {} {}: {} ({}) {}'.format(color, origen, flecha, destino, movement['event']['missionText'], daysHoursMinutes(tiempoFaltante), bcolors.ENDC))

			if movement['isHostile']:
				tropas = movement['army']['amount']
				flotas = movement['fleet']['amount']
				print(_('Troops:{}\nFleets:{}').format(addDot(tropas), addDot(flotas)))
			elif esHostil(movement):
				tropas = movement['army']['amount']
				barcos = 0
				flotas = 0
				for mov in movement['fleet']['ships']:
					if mov['cssClass'] == 'ship_transport':
						barcos += int(mov['amount'])
					else:
						flotas += int(mov['amount'])
				print(_('Troops:{}\nFleets:{}\n Ships:{}').format(addDot(tropas), addDot(flotas), addDot(barcos)))
			else:
				bien = {'wood': _('wood'), 'wine': _('wine'), 'marble': _('marble'), 'glass': _('cristal'), 'sulfur': _('sulfur')}
				cargaTotal = 0
				for resource in movement['resources']:
					cantidad = resource['amount']
					tipo = resource['cssClass'].split()[1]
					tipo = bien[tipo]
					cargaTotal += int( cantidad.replace(',', '') )
					print(_('{} of {}').format(cantidad, tipo))
				barcos = int(math.ceil((Decimal(cargaTotal) / Decimal(500))))
				print(_('{:d} Ships').format(barcos))
		enter()
		e.set()
	except KeyboardInterrupt:
		e.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import gettext
from ikabot.config import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.getJson import *
from ikabot.helpers.recursos import *
from ikabot.helpers.gui import *
from ikabot.helpers.varios import addDot

t = gettext.translation('donate', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def donate(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		bienes = {'1': _('Vineyard'), '2': _('Quarry'), '3': _('Crystal Mine'), '4': _('Sulfur Pit')}
		banner()

		ciudad = chooseCity(s)
		banner()

		madera = ciudad['recursos'][0]
		almacenamiento = ciudad['storageCapacity']

		idIsla = ciudad['islandId']
		html = s.get(urlIsla + idIsla)
		isla = getIsla(html)

		tipo = isla['tipo']
		bien = bienes[tipo]

		urlAserradero = 'view=resource&type=resource&islandId={0}&backgroundView=island&currentIslandId={0}&actionRequest={1}&ajax=1'.format(idIsla, s.token())
		aserraderoOk = printEstadoMina(s, urlAserradero, 'Aserradero')

		urlBien = 'view=tradegood&type={0}&islandId={1}&backgroundView=island&currentIslandId={1}&actionRequest={2}&ajax=1'.format(tipo, idIsla, s.token())
		bienOk = printEstadoMina(s, urlBien, bien)

		tipo = ['resource', 'tradegood']
		print(_('Wood available:{} / {}\n').format(addDot(madera), addDot(almacenamiento)))

		if aserraderoOk is True and bienOk is True:
			msg = _('Forest(1) o {}(2)?:').format(bien)
			tipoDonacion = read(msg=msg, min=1, max=2)
		elif aserraderoOk is True and bienOk is False:
			tipoDonacion = 1
			print(_('Forest:\n'))
		elif aserraderoOk is False and bienOk is True:
			tipoDonacion = 2
			print('{}:\n'.format(bien))
		else:
			print(_('You cannot donate\n'))
			return

		tipo = tipo[tipoDonacion - 1]

		cantidad = read(min=0, max=madera, msg=_('Amount:'))
		s.post(payloadPost={'islandId': idIsla, 'type': tipo, 'action': 'IslandScreen', 'function': 'donate', 'donation': cantidad, 'backgroundView': 'island', 'templateView': 'resource', 'actionRequest': s.token(), 'ajax': '1'})
		e.set()
	except KeyboardInterrupt:
		e.set()
		return

def printEstadoMina(s, url, bien):
	html = s.post(url)
	levels = re.search(r'"resourceLevel":"(\d+)","tradegoodLevel":"(\d+)"', html)
	if bien == _('Forest'):
		lv = levels.group(1)
	else:
		lv = levels.group(2)
	infoMina = re.search(r':<\\/h4>\\n\s*<ul\sclass=\\"resources\\">\\n\s*<li\sclass=\\"wood\\">([\d,]+)<[\s\S]*?:<\\/h4>\\n\s*<ul\sclass=\\"resources\\">\\n\s*<li\sclass=\\"wood\\">([\d,]+)<', html)
	if infoMina is not None:
		donado = infoMina.group(2)
		pordonate = infoMina.group(1)
		donado = int(donado.replace(',', ''))
		pordonate = int(pordonate.replace(',', ''))
		print('{} lv:{}'.format(bien, lv))
		print('{} / {} {}%'.format(addDot(donado), addDot(pordonate), addDot(int((100 * donado) / pordonate))))
	else:
		print(_('{}: Is expanding to level {:d}\n').format(bien, int(lv) + 1))
	return infoMina is not None



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
from decimal import *

getcontext().prec = 30

def getRecursosDisponibles(html, num=False):
	recursos = re.search(r'\\"resource\\":(\d+),\\"2\\":(\d+),\\"1\\":(\d+),\\"4\\":(\d+),\\"3\\":(\d+)}', html)
	if num:
		return [int(recursos.group(1)), int(recursos.group(3)), int(recursos.group(2)), int(recursos.group(5)), int(recursos.group(4))]
	else:
		return [recursos.group(1), recursos.group(3), recursos.group(2), recursos.group(5), recursos.group(4)]

def getstorageCapacityDeAlmacenamiento(html):
	almacenamiento = re.search(r'maxResources:\s*JSON\.parse\(\'{\\"resource\\":(\d+),', html).group(1)
	return int(almacenamiento)

def getConsumoDeVino(html):
	rta = re.search(r'GlobalMenu_WineConsumption"\s*class="rightText">\s*(\d+)\s', html)
	if rta:
		return int(rta.group(1))
	return 0

def getProduccionPerSecond(s, idCiudad):
	prod = s.post(payloadPost={'action': 'header', 'function': 'changeCurrentCity', 'actionRequest': s.token(), 'cityId': idCiudad, 'ajax': '1'})
	prod = json.loads(prod, strict=False)
	prod = prod[0][1]['headerData']
	wood = Decimal( prod['resourceProduction'] )
	good = Decimal( prod['tradegoodProduction'] )
	typeGood = int( prod['producedTradegood'] )
	return (wood, good, typeGood)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import json
from ikabot.helpers.recursos import *

def borrar(texto, ocurrencias):
	for ocurrencia in ocurrencias:
		texto = texto.replace(ocurrencia, '')
	return texto

def getCiudadanosDisponibles(html):
	ciudadanosDisp = re.search(r'js_GlobalMenu_citizens">(.*?)</span>', html).group(1)
	return int(ciudadanosDisp.replace(',', '').replace('.', ''))

def enVenta(html):
	rta = re.search(r'branchOfficeResources: JSON\.parse\(\'{\\"resource\\":\\"(\d+)\\",\\"1\\":\\"(\d+)\\",\\"2\\":\\"(\d+)\\",\\"3\\":\\"(\d+)\\",\\"4\\":\\"(\d+)\\"}\'\)', html)
	if rta:
		return [ int(rta.group(1)), int(rta.group(2)), int(rta.group(3)), int(rta.group(4)), int(rta.group(5))]
	else:
		return [0, 0, 0, 0, 0]

def getIsla(html):
	isla = re.search(r'\[\["updateBackgroundData",([\s\S]*?),"specialServerBadges', html).group(1) + '}'

	isla = isla.replace('buildplace', 'empty')
	isla = isla.replace('xCoord', 'x')
	isla = isla.replace('yCoord', 'y')
	isla = isla.replace(',"owner', ',"')
	isla = isla.replace(',"tradegoodLevel',',"goodLv')
	isla = isla.replace(',"tradegood', ',"good')
	isla = isla.replace(',"resourceLevel', ',"woodLv')
	isla = isla.replace(',"wonderLevel', ',"wonderLv')
	isla = isla.replace('avatarScores', 'scores')

	remove = []

	sub = re.search(r',"type":\d', isla).group()
	remove.append(sub)

	quitar = re.search(r'(,"barbarians[\s\S]*?),"scores"', isla).group(1) #to remove

	remove.append(quitar) #to remove
	remove.append(',"goodTarget":"tradegood"')
	remove.append(',"name":"Building ground"')
	remove.append(',"name":"Terreno"')
	remove.append(',"actions":[]') #to remove
	remove.append('"id":-1,')
	remove.append(',"level":0,"viewAble":1')
	remove.append(',"empty_type":"normal"')
	remove.append(',"empty_type":"premium"')
	remove.append(',"hasTreaties":0')
	remove.append(',"hasTreaties":1')
	remove.append(',"infestedByPlague":false')
	remove.append(',"infestedByPlague":true')
	remove.append(',"viewAble":0')
	remove.append(',"viewAble":1')
	remove.append(',"viewAble":2')
	isla = borrar(isla, remove)

	# {"id":idIsla,"name":nombreIsla,"x":,"y":,"good":numeroBien,"woodLv":,"goodLv":,"wonder":numeroWonder, "wonderName": "nombreDelMilagro","wonderLv":"5","cities":[{"type":"city","name":cityName,"id":cityId,"level":lvIntendencia,"Id":playerId,"Name":playerName,"AllyId":,"AllyTag":,"state":"vacation"},...}}
	isla = json.loads(isla, strict=False)
	tipo = re.search(r'"tradegood":"(\d)"', html).group(1)
	isla['tipo'] = tipo
	return isla

def getCiudad(html):

	ciudad = re.search(r'"updateBackgroundData",\s?([\s\S]*?)\],\["updateTemplateData"', html).group(1)
	ciudad = json.loads(ciudad, strict=False)

	ciudad['Id'] = ciudad.pop('ownerId')
	ciudad['Name'] = ciudad.pop('ownerName')
	ciudad['x'] = ciudad.pop('islandXCoord')
	ciudad['y'] = ciudad.pop('islandYCoord')
	ciudad['cityName'] = ciudad['name']

	i = 0
	for position in ciudad['position']:
		position['position'] = i
		i += 1
		if 'level' in position:
			position['level'] = int(position['level'])
		position['isBusy'] = False
		if 'constructionSite' in position['building']:
			position['isBusy'] = True
			position['building'] = position['building'][:-17]
		elif 'buildingGround ' in position['building']:
			position['name'] = 'empty'
			position['type'] = position['building'].split(' ')[-1]
			position['building'] = 'empty'

	ciudad['id'] = str(ciudad['id'])
	ciudad['propia'] = True
	ciudad['recursos'] = getRecursosDisponibles(html, num=True)
	ciudad['storageCapacity'] = getstorageCapacityDeAlmacenamiento(html)
	ciudad['ciudadanosDisp'] = getCiudadanosDisponibles(html)
	ciudad['consumo'] = getConsumoDeVino(html)
	ciudad['enventa'] = enVenta(html)
	ciudad['freeSpaceForResources'] = []
	for i in range(5):
		ciudad['freeSpaceForResources'].append( ciudad['storageCapacity'] - ciudad['recursos'][i] - ciudad['enventa'][i] )

	return ciudad



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import gettext
import sys
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.process import run
from ikabot.helpers.gui import *
from ikabot.config import *

t = gettext.translation('update', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def update(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		print(_('To update ikabot run:'))
		print('python3 -m pip install --user --upgrade ikabot')
		enter()
		e.set()
	except KeyboardInterrupt:
		e.set()
		return



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import gettext
import traceback
from ikabot.config import *
from ikabot.helpers.botComm import *
from ikabot.helpers.pedirInfo import *
from ikabot.helpers.gui import *
from ikabot.helpers.process import set_child_mode
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.recursos import getRecursosDisponibles

t = gettext.translation('donationBot', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

def donationBot(s,e,fd):
	sys.stdin = os.fdopen(fd)
	try:
		banner()
		(idsCiudades, ciudades) = getIdsOfCities(s)
		ciudades_dict = {}
		bienes = {'1': _('(W)'), '2': _('(M)'), '3': _('(C)'), '4': _('(S)')}
		for idCiudad in idsCiudades:
			tradegood = ciudades[idCiudad]['tradegood']
			bien = bienes[tradegood]
			print(_('In the city {} {}, Do you wish to donate to the forest, to the trading good or neither? [f/t/n]').format(ciudades[idCiudad]['name'], bien))
			rta = read(values=[_('f'), _('F'), _('t'), _('T'), _('n'), _('N')])
			if rta.lower() == _('a'):
				tipo = 'resource'
			elif rta.lower() == _('b'):
				tipo = 'tradegood'
			else:
				tipo = None
			ciudades_dict[idCiudad] = {'tipo': tipo}

		print(_('I will donate every day.'))
		enter()
	except KeyboardInterrupt:
		e.set()
		return

	set_child_mode(s)
	e.set()

	info = _('\nI donate every day\n')
	setInfoSignal(s, info)
	try:
		do_it(s, idsCiudades, ciudades_dict)
	except:
		msg = _('Error in:\n{}\nCause:\n{}').format(info, traceback.format_exc())
		sendToBot(s, msg)
	finally:
		s.logout()

def do_it(s, idsCiudades, ciudades_dict):
	for idCiudad in idsCiudades:
		html = s.get(urlCiudad + idCiudad)
		ciudad = getCiudad(html)
		ciudades_dict[idCiudad]['isla'] = ciudad['islandId']
	while True:
		for idCiudad in idsCiudades:
			html = s.get(urlCiudad + idCiudad)
			madera = getRecursosDisponibles(html)[0]
			idIsla = ciudades_dict[idCiudad]['isla']
			tipo = ciudades_dict[idCiudad]['tipo']
			if tipo:
				s.post(payloadPost={'islandId': idIsla, 'type': tipo, 'action': 'IslandScreen', 'function': 'donate', 'donation': madera, 'backgroundView': 'island', 'templateView': tipo, 'actionRequest': s.token(), 'ajax': '1'})
		msg = _('I donated automatically.')
		sendToBotDebug(s, msg, debugON_donationBot)
		time.sleep(24*60*60)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import re
import sys
import json
import gettext
from decimal import *
from ikabot.config import *
from ikabot.helpers.getJson import *
from ikabot.helpers.gui import *

t = gettext.translation('pedirInfo', 
                        localedir, 
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

getcontext().prec = 30

def read(min=None, max=None, digit=False, msg=prompt, values=None, empty=False): # lee input del usuario
	"""Reads input from user
	Parameters
	----------
	min : int
		smallest number acceptable as input
	max : int
		greatest number acceptable as input
	digit : bool
		boolean indicating whether or not the input MUST be an int
	msg : str
		string printed before the user is asked for input
	values : list
		list of strings which are acceptable as input
	empty : bool
		a boolean indicating whether or not an empty string is acceptable as input

	Returns
	-------
	result : str
		string representing the user's input
	"""
	def _invalido():
		print('\033[1A\033[K', end="") # Borro linea
		return read(min, max, digit, msg, values)

	try:
		leido = input(msg)
	except EOFError:
		return _invalido()

	if leido == '' and empty is True:
		return leido

	if digit is True or min is not None or max is not None:
		if leido.isdigit() is False:
			return _invalido()
		else:
			try:
				leido = eval(leido)
			except SyntaxError:
				return _invalido()
	if min is not None and leido < min:
		return _invalido()
	if max is not None and leido > max:
		return _invalido()
	if values is not None and leido not in values:
		return _invalido()
	return leido

def chooseCity(s, foreign=False):
	"""Prompts the user to chose a city
	Parameters
	----------
	s : Session
		Session object
	foreign : bool
		lets the user choose a foreign city
	
	Returns
	-------
	city : City 
		a city object representing the chosen city
	"""
	global menuCiudades
	(ids, ciudades) = getIdsOfCities(s)
	if menuCiudades == '':
		maxNombre = 0
		for unId in ids:
			largo = len(ciudades[unId]['name'])
			if largo > maxNombre:
				maxNombre = largo
		pad = lambda name: ' ' * (maxNombre - len(name) + 2)
		bienes = {'1': _('(W)'), '2': _('(M)'), '3': _('(C)'), '4': _('(S)')}
		prints = []
		i = 0
		if foreign:
			print(_(' 0: foreign city'))
		else:
			print('')
		for unId in ids:
			i += 1
			tradegood = ciudades[unId]['tradegood']
			bien = bienes[tradegood]
			nombre = ciudades[unId]['name']
			matches = re.findall(r'u[0-9a-f]{4}', nombre)
			for match in matches:
				to_unicode = '\\' + match
				to_unicode = to_unicode.encode().decode('unicode-escape')
				nombre = nombre.replace(match, to_unicode)
			num = ' ' + str(i) if i < 10 else str(i)
			menuCiudades += '{}: {}{}{}\n'.format(num, nombre, pad(nombre), bien)
		menuCiudades = menuCiudades[:-1]
	if foreign:
		print(_(' 0: ciudad ajena'))
	print(menuCiudades)

	if foreign:
		eleccion = read(min=0, max=len(ids))
	else:
		eleccion = read(min=1, max=len(ids))
	if eleccion == 0:
		return chooseForeignCity(s)
	else:
		html = s.get(urlCiudad + ids[eleccion -1])
		return getCiudad(html)

def chooseForeignCity(s):
	"""Prompts the user to select an island, and a city on that island (is only used in chooseCity)
	Parameters
	----------
	s : Session
		Session object

	Returns
	-------
	 city : City
		a city object representing the city the user chose
	"""
	banner()
	x = read(msg='coordenada x:', digit=True)
	y = read(msg='coordenada y:', digit=True)
	print('')
	url = 'view=worldmap_iso&islandX={}&islandY={}&oldBackgroundView=island&islandWorldviewScale=1'.format(x, y)
	html = s.get(url)
	try:
		jsonIslas = re.search(r'jsonData = \'(.*?)\';', html).group(1)
		jsonIslas = json.loads(jsonIslas, strict=False)
		idIsla = jsonIslas['data'][str(x)][str(y)][0]
	except:
		print(_('Incorrect coordinates'))
		enter()
		banner()
		return chooseCity(s, foreign=True)
	html = s.get(urlIsla + idIsla)
	isla = getIsla(html)
	maxNombre = 0
	for ciudad in isla['cities']:
		if ciudad['type'] == 'city':
			largo = len(ciudad['name'])
			if largo > maxNombre:
				maxNombre = largo
	pad = lambda name: ' ' * (maxNombre - len(name) + 2)
	i = 0
	opciones = []
	for ciudad in isla['cities']:
		if ciudad['type'] == 'city' and ciudad['state'] == '' and ciudad['Name'] != s.username:
			i += 1
			num = ' ' + str(i) if i < 10 else str(i)
			print('{}: {}{}({})'.format(num, ciudad['name'], pad(ciudad['name']), ciudad['Name']))
			opciones.append(ciudad)
	if i == 0:
		print(_('There are no cities where to send resources on this island'))
		enter()
		return chooseCity(s, foreign=True)
	eleccion = read(min=1, max=i)
	ciudad = opciones[eleccion - 1]
	ciudad['islandId'] = isla['id']
	ciudad['cityName'] = ciudad['name']
	ciudad['propia'] = False
	return ciudad

def askForValue(text, max):
	"""Displays text and asks the user to enter a value between 0 and max

	Parameters
	----------
	text : str
		text to be displayed when asking the user for input
	max : int
		integer representing the number of input options
	
	Returns
	-------
	var : int
		integer representing the user's input
		if the user has inputed nothing, 0 will be returned instead
	"""
	var = read(msg=text, min=0, max=max, empty=True)
	if var == '':
		var = 0
	return var

def getIdsOfCities(s, all=False):
	"""Gets the user's cities
	Parameters
	----------
	s : Session
		Session object
	all : bool
		boolean indicating whether all cities should be returned, or only those that belong to the current user

	Returns
	-------
	(ids, cities) : tuple
		a tuple containing the a list of city IDs and a list of city objects
	"""
	global ciudades
	global ids
	if ids is None or ciudades is None or s.padre is False:
		html = s.get()
		ciudades = re.search(r'relatedCityData:\sJSON\.parse\(\'(.+?),\\"additionalInfo', html).group(1) + '}'
		ciudades = ciudades.replace('\\', '')
		ciudades = ciudades.replace('city_', '')
		ciudades = json.loads(ciudades, strict=False)

		ids = [ciudad for ciudad in ciudades]
		ids = sorted(ids)

	# {'coords': '[x:y] ', 'id': idCiudad, 'tradegood': '..', 'name': 'nomberCiudad', 'relationship': 'ownCity'|'occupiedCities'|..}
	if all is False:
		ids_own   = [ciudad for ciudad in ciudades if ciudades[ciudad]['relationship'] == 'ownCity']
		ids_other = [ciudad for ciudad in ciudades if ciudades[ciudad]['relationship'] != 'ownCity']
		ciudades_own = ciudades.copy()
		for id in ids_other:
			del ciudades_own[id]
		return (ids_own, ciudades_own)
	else:
		return (ids, ciudades)

def getIdsOfIslands(s):
	"""Gets the IDs of islands the user has cities on
	Parameters
	----------
	s : Session
		Session object
	
	Returns
	-------
	idsIslas : list
		a list containing the IDs of the users islands
	"""
	(idsCiudades, ciudades) = getIdsOfCities(s)
	idsIslas = set()
	for idCiudad in idsCiudades:
		html = s.get(urlCiudad + idCiudad)
		ciudad = getCiudad(html)
		idIsla = ciudad['islandId']
		idsIslas.add(idIsla)
	return list(idsIslas)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import json
import base64
import hashlib
from ikabot.config import *
from ikabot.helpers.botComm import *
from cryptography.hazmat.primitives.ciphers.aead import AESGCM

t = gettext.translation('aesCipher',
                        localedir,
                        languages=idiomas,
                        fallback=True)
_ = t.gettext

class AESCipher:

	def __init__( self, mail, username, password ):
		self.key = hashlib.sha256( mail.encode('utf-8') + b'\x00' + password.encode('utf-8') ).digest()
		for i in range(0xfff):
			self.key = hashlib.sha256( self.key ).digest()

	def encrypt( self, plaintext ):
		aesgcm = AESGCM(self.key)
		nonce = os.urandom(16)
		ciphertext = aesgcm.encrypt(nonce, plaintext.encode('utf-8'), None)
		return base64.b64encode( nonce + ciphertext ).decode('utf-8')

	def decrypt( self, ciphertext ):
		ciphertext = base64.b64decode(ciphertext)
		nonce      = ciphertext[:16]
		ciphertext = ciphertext[16:]
		aesgcm = AESGCM(self.key)
		plaintext  = aesgcm.decrypt(nonce, ciphertext, None)
		return plaintext.decode('utf-8')

	def getEntryKey(self, s):
		return hashlib.sha256( 'ikabot'.encode('utf-8') + s.mail.encode('utf-8') ).hexdigest()

	def deleteSessionData(self, s):
		entry_key = self.getEntryKey(s)
		with open(ikaFile, 'r') as filehandler:
			data = filehandler.read()

		newFile = ''
		for line in data.split('\n'):
			if entry_key != line[:64]:
				newFile += line + '\n'

		with open(ikaFile, 'w') as filehandler:
			filehandler.write(newFile.strip())
			filehandler.flush()

	def getSessionData(self, s, all=False):
		entry_key = self.getEntryKey(s)
		with open(ikaFile, 'r') as filehandler:
			ciphertexts = filehandler.read()

		for ciphertext in ciphertexts.split('\n'):
			if entry_key == ciphertext[:64]:
				ciphertext = ciphertext[64:]
				try:
					plaintext = self.decrypt(ciphertext)
				except:
					msg = _('Error while decrypting session data\nSaved data will be deleted.')
					if s.padre:
						print(msg)
					else:
						sendToBot(s, msg)
					self.deleteSessionData(s)
					os._exit(0)
				data_dict = json.loads(plaintext, strict=False)
				if all:
					return data_dict
				else:
					try:
						return data_dict[s.username][s.mundo][s.servidor]
					except KeyError:
						return {}
		return {}

	def setSessionData(self, s, data):
		session_data = self.getSessionData(s, True)

		if s.username not in session_data:
			session_data[s.username] = {}
		if s.mundo not in session_data[s.username]:
			session_data[s.username][s.mundo] = {}
		if s.servidor not in session_data[s.username][s.mundo]:
			session_data[s.username][s.mundo][s.servidor] = {}

		session_data[s.username][s.mundo][s.servidor] = data

		plaintext  = json.dumps(session_data)
		ciphertext = self.encrypt(plaintext)

		with open(ikaFile, 'r') as filehandler:
			data = filehandler.read()

		entry_key  = self.getEntryKey(s)
		newFile = ''
		newline = entry_key + ' ' + ciphertext
		for line in data.split('\n'):
			if entry_key != line[:64]:
				newFile += line + '\n'
		newFile += newline + '\n'

		with open(ikaFile, 'w') as filehandler:
			filehandler.write(newFile.strip())
			filehandler.flush()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import random
from decimal import *

getcontext().prec = 30

def addDot(num):
	"""Formats the number into a string and adds a '.' for every thousand (eg. 3000 -> 3.000)
	Parameters
	----------
	num : int
		integer number to format
	
	Returns
	-------
	number : str
		a string representing that number with added dots for every thousand
	"""
	return '{0:,}'.format(int(num)).replace(',','.')

def daysHoursMinutes(totalSeconds):
	"""Formats the total number of seconds into days hours minutes (eg. 321454 -> 3D 17H) 
	Parameters
	----------
	totalSeconds : int
		total number of seconds
	
	Returns
	-------
	text : str
		formatted string (D H M)
	"""
	if totalSeconds == 0:
		return '0 s'
	dias = int(totalSeconds / Decimal(86400))
	totalSeconds -= dias * Decimal(86400)
	horas = int(totalSeconds / Decimal(3600))
	totalSeconds -= horas * Decimal(3600)
	minutos = int(totalSeconds / Decimal(60))
	texto = ''
	if dias > 0:
		texto = str(dias) + 'D '
	if horas > 0:
		texto = texto + str(horas) + 'H '
	if minutos > 0 and dias == 0:
		texto = texto + str(minutos) + 'M '
	return texto[:-1]

def wait(seconds, maxrandom = 0):
	"""This function will wait the provided number of seconds plus a random number of seconds between 0 and maxrandom
	Parameteres
	-----------
	seconds : int
		the number of seconds to wait for
	maxrandom : int
		the maximum number of additional seconds to wait for
	"""
	randomTime = random.randint(0, maxrandom)
	if seconds <= 0:
		return
	ratio = (1 + 5 ** 0.5) / 2 - 1 # 0.6180339887498949
	comienzo = time.time()
	fin = comienzo + seconds
	restantes = seconds
	while restantes > 0:
		time.sleep(restantes * ratio)
		restantes = fin - time.time()
	time.sleep(randomTime)
	return
